
//		SoundVolMain.cpp - plugin to get and set sound volume and mute state


#define UNICODE					1
#define _UNICODE				1

#include <stdio.h>
#include <tchar.h>

#include <windows.h>

#include "PlugIn.h"
#include "TakeCmd.h"


#define VER_MAJOR				1
#define VER_MINOR				0
#define VER_BUILD				0
#define VER_PATCH				2

#define PLUGIN_NAME				L"SoundVol"

#ifdef _WIN64
#define PLATFORM				L"x64"
#else
#define PLATFORM				L"x86"
#endif

#define MAX_ARG_LEN				8192
#define SHORT_BUF_LEN			1024
#define MAX_FILENAME_LEN		32768

#define ERRHANDLE				GetStdHandle( STD_ERROR_HANDLE )


#define MACRO_EXP( arg )		#arg
#define MACRO_STR( arg )		MACRO_EXP( arg )


HINSTANCE MyDLLHandle			= NULL;


OSVERSIONINFOEX OSVerInfo;



const wchar_t PluginFeatures[] = L"soundvol,_mute,_volume";



int DisplayErrorHeader()
{
	wchar_t shortbuf[MAX_ARG_LEN] = L"%_batch";

	Qprintf( ERRHANDLE, L"%s plugin: ", PLUGIN_NAME );

	ExpandVariables( shortbuf, 1 );
	if ( shortbuf[0] == '0' )
		return 0;

	wcscpy_s( shortbuf, MAX_ARG_LEN, L"%_batchname [%_batchline]  " );
	ExpandVariables( shortbuf, 1 );
	Qprintf( ERRHANDLE, shortbuf );

	return 0;
}



int ParseInt( LPTSTR inString, unsigned int *outValue, wchar_t **EndChar )
{
	if ( !inString || !outValue )
		return -1;

	unsigned int value	= 0;
	wchar_t *sep1		= NULL;

	_set_errno( 0 );

	if ( ( inString[0] == '0' ) && ( towupper( inString[1] ) == 'X' ) && ( iswxdigit( inString[2] ) ) ) {
		//  parse hexadecimal

		value = wcstoul( inString + 2, &sep1, 16 );

		if ( errno )
			return errno;
		else if ( !EndChar && *sep1 )
			return -1;

		*outValue = value;

		if ( EndChar )
			*EndChar  = sep1;

		return 0;
	}
	else if ( iswdigit( inString[0] ) ) {
		//  parse unsigned

		value = wcstoul( inString, &sep1, 10 );

		if ( errno )
			return errno;
		else if ( !EndChar && *sep1 )
			return -1;

		*outValue = value;

		if ( EndChar )
			*EndChar  = sep1;

		return 0;
	}

	return -1;
}



LPCTSTR _FilenamePart( LPCTSTR Filename, LPTSTR outFilename )
{
	if ( !Filename )
		return NULL;
	else if ( Filename[0] == '\0' )
		return NULL;


	BOOL FoundSlash			= FALSE;
	size_t EndOfPathPart	= -1;


	for ( size_t i = 0; Filename[i]; i++ ) {
		if ( ( Filename[i] == '/' ) || ( Filename[i] == '\\' ) ) {
			EndOfPathPart = i;
			FoundSlash = TRUE;
		}
		else if ( ( Filename[i] == ':' ) && !FoundSlash )
			EndOfPathPart = i;
	}

	if ( EndOfPathPart == -1 ) {
		if ( outFilename ) {
			wcscpy_s( outFilename, MAX_FILENAME_LEN, Filename );
			return outFilename;
		}
		else
			return Filename;
	}

	if ( outFilename ) {
		wcscpy_s( outFilename, MAX_FILENAME_LEN, Filename + EndOfPathPart + 1 );
		return outFilename;
	}
	else
		return Filename + EndOfPathPart + 1;
}



int ShowCmdHelp( LPCTSTR HelpText, BOOL ShowVersion, LPCSTR SourceFile, LPCSTR SourceFileVer )
{
	wchar_t File[MAX_FILENAME_LEN]		= L"";
	wchar_t Ver[SHORT_BUF_LEN]			= L"";


	QPuts( (LPTSTR) HelpText );

	if ( ShowVersion ) {
		Printf( L"\r\n\t\t\t\t%s  %u.%u.%u.%u (%s)\r\n", PLUGIN_NAME, VER_MAJOR, VER_MINOR, VER_BUILD, VER_PATCH, PLATFORM );

		if ( SourceFile && SourceFileVer ) {
			MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, SourceFile, -1, File, MAX_FILENAME_LEN );
			MultiByteToWideChar( CP_ACP, MB_PRECOMPOSED, SourceFileVer, -1, Ver, SHORT_BUF_LEN );
			Printf( L"\t\t\t\t%s  %s\r\n", _FilenamePart( File, NULL), Ver );
		}
	}

	return 0;
}



#include "ParseArgs.cpp"
#include "OsdCmd.cpp"

#define  SOUNDVOL_CMD
#define  VOLUME_VAR
#define  VOLUME_VAR_HELP_TEXT
#include "SoundVol.cpp"



BOOL GetVersion2( OSVERSIONINFOEXW *VerInfo )
{
	if ( !VerInfo )
		return -1;


	typedef NTSTATUS ( WINAPI _RtlGetVersionProc ) ( OSVERSIONINFOEXW *VerInfo );
	HMODULE NtDllHandle						= GetModuleHandle( L"NtDll.dll" );
	_RtlGetVersionProc *RtlGetVersionProc	= NULL;


	memset( VerInfo, 0x00, sizeof( OSVERSIONINFOEXW ) );
	VerInfo->dwOSVersionInfoSize = sizeof( OSVERSIONINFOEXW );

	if ( NtDllHandle )
		RtlGetVersionProc = (_RtlGetVersionProc*) GetProcAddress( NtDllHandle, "RtlGetVersion" );

	if ( RtlGetVersionProc ) {
		RtlGetVersionProc( VerInfo );
		if ( VerInfo->dwMajorVersion >= 6 )
			return TRUE;
	}

	memset( VerInfo, 0x00, sizeof( OSVERSIONINFOEXW ) );
	VerInfo->dwOSVersionInfoSize = sizeof( OSVERSIONINFOEXW );

	return ( GetVersionEx( (OSVERSIONINFOW*) VerInfo ) != 0 );
}



BOOL APIENTRY DllMain( HANDLE hModule, DWORD  dwReason, LPVOID lpReserved )
{
	switch( dwReason )	{
		case DLL_PROCESS_ATTACH:
			MyDLLHandle = (HINSTANCE) hModule;
			break;

		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
	}
	return TRUE;
}



DLLExports LPPLUGININFO WINAPI GetPluginInfo( void )
{
	static PLUGININFO piInfo;

	piInfo.pszDll			= (LPTSTR) PLUGIN_NAME;
	piInfo.pszAuthor		= (LPTSTR) L"Charles Dye";
	piInfo.pszEmail			= (LPTSTR) L"cdye@unm.edu";
	piInfo.pszWWW			= (LPTSTR) L"http://charlesdye.net/plugins/soundvol.html";
	piInfo.pszDescription	= (LPTSTR) L"Get and set sound volume and mute state.";
	piInfo.pszFunctions		= (LPTSTR) PluginFeatures;
	piInfo.nMajor			= VER_MAJOR;
	piInfo.nMinor			= VER_MINOR;
	piInfo.nBuild			= VER_BUILD;

	return &piInfo;
}



DLLExports BOOL WINAPI InitializePlugin( void )
{
	wchar_t shortbuf[SHORT_BUF_LEN] = L"%[_transient]%[_pipe]%[noloadmsg]";
	ExpandVariables( shortbuf, 1 );

	GetVersion2( &OSVerInfo );

#ifdef OSD_VERSION

	OSD_Initialize();

#endif	//	OSD_VERSION

	if ( !wcscmp( shortbuf, L"00" ) )
		Printf( L"%s v%u.%u.%u loaded.\r\n", PLUGIN_NAME, VER_MAJOR, VER_MINOR, VER_BUILD );

	return 0;
}



DLLExports BOOL WINAPI ShutdownPlugin( BOOL bEndProcess )
{

#ifdef OSD_VERSION

	OSD_Shutdown();

#endif	OSD_VERSION

	return 0;
}



const int NumHelpTexts							= 3;
const wchar_t HelpFeature[NumHelpTexts][11]		= { L"soundvol", L"_mute", L"_volume" };
const wchar_t *HelpText[NumHelpTexts]			= { SoundVolHelpText, _MuteHelpText, _VolumeHelpText };



LPTSTR UsageText( const wchar_t *inString )
{
	if ( !inString )
		return NULL;


	wchar_t *ch = wcschr( (wchar_t*) inString, '\n' );


	if ( !ch )
		return NULL;

	while ( *ch == '\n' )
		ch++;

	return ch;
}



DLLExports int WINAPI Usage( LPTSTR inString, LPTSTR outString )
{
	if ( ( inString[0] == '@' ) || ( inString[0] == '_' ) )
		return 0;

	for ( int i = 0; i < NumHelpTexts; i++ ) {
		if ( !_wcsicmp( inString, HelpFeature[i] ) ) {
			wcscpy_s( outString, SHORT_BUF_LEN, UsageText( HelpText[i] ) );
			return 1;
		}
	}

	return 0;
}


//		Simpler MessageBox() help, useful for smaller plugins which don't merit a full HTML help system:

DLLExports int WINAPI Help( LPTSTR inString )
{
	wchar_t WindowTitle[SHORT_BUF_LEN]		= L"";


	wsprintf( WindowTitle, L"%s Plugin Help", PLUGIN_NAME );

	for ( int i = 0; i < NumHelpTexts; i++ ) {
		if ( !_wcsicmp( inString, HelpFeature[i] ) ) {
			MessageBox( GetForegroundWindow(), HelpText[i], WindowTitle, MB_OK );
			return 1;
		}
	}

	return 0;
}



/*

	0.94.0		2022-12-27

	0.95.0		2023-02-01
				SOUNDVOL now displays the friendly name of the default output device.
				(Vista and later only; does anyone still use XP?)

	0.96.0		2023-02-01
				Split the source:  The plugin body is now SoundVolMain.cpp; the sound
				volume stuff has been moved out to SoundVol.cpp.

*/
